@@ -1,5 +1,8 @@
 //! Commonly used listeners.
 
+#[cfg(feature = "acme")]
+#[cfg_attr(docsrs, doc(cfg(feature = "acme")))]
+pub mod acme;
 mod combined;
 #[cfg(any(feature = "native-tls", feature = "rustls"))]
 mod handshake_stream;
@@ -20,21 +23,25 @@ use std::{
     task::{Context, Poll},
 };
 
-pub use combined::{Combined, CombinedStream};
-#[cfg(any(feature = "native-tls", feature = "rustls"))]
-pub use handshake_stream::HandshakeStream;
 use http::uri::Scheme;
-#[cfg(feature = "native-tls")]
-pub use native_tls::{NativeTlsAcceptor, NativeTlsConfig, NativeTlsListener};
-#[cfg(feature = "rustls")]
-pub use rustls::{RustlsAcceptor, RustlsConfig, RustlsListener};
-pub use tcp::{TcpAcceptor, TcpListener};
-#[cfg(any(feature = "rustls", feature = "native-tls"))]
-pub use tls::IntoTlsConfigStream;
 use tokio::io::{AsyncRead, AsyncWrite, ReadBuf, Result as IoResult};
-#[cfg(unix)]
-pub use unix::{UnixAcceptor, UnixListener};
 
+#[cfg(feature = "acme")]
+use self::acme::{AutoCert, AutoCertListener};
+#[cfg(any(feature = "native-tls", feature = "rustls"))]
+pub use self::handshake_stream::HandshakeStream;
+#[cfg(feature = "native-tls")]
+pub use self::native_tls::{NativeTlsAcceptor, NativeTlsConfig, NativeTlsListener};
+#[cfg(feature = "rustls")]
+pub use self::rustls::{RustlsAcceptor, RustlsConfig, RustlsListener};
+#[cfg(any(feature = "rustls", feature = "native-tls"))]
+pub use self::tls::IntoTlsConfigStream;
+#[cfg(unix)]
+pub use self::unix::{UnixAcceptor, UnixListener};
+pub use self::{
+    combined::{Combined, CombinedStream},
+    tcp::{TcpAcceptor, TcpListener},
+};
 use crate::web::{LocalAddr, RemoteAddr};
 
 /// Represents a acceptor type.
@@ -69,7 +76,7 @@ pub trait AcceptorExt: Acceptor {
         Combined::new(self, other)
     }
 
-    /// Wrap the acceptor in a Box.
+    /// Wrap the acceptor in a `Box`.
     fn boxed(self) -> BoxAcceptor
     where
         Self: Sized + 'static,
@@ -157,8 +164,48 @@ pub trait Listener: Send {
     {
         NativeTlsListener::new(self, config_stream)
     }
+
+    /// Consume this listener and return a new ACME listener.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use poem::listener::{
+    ///     acme::{AutoCert, LETS_ENCRYPT_PRODUCTION},
+    ///     Listener, TcpListener,
+    /// };
+    ///
+    /// let listener = TcpListener::bind("0.0.0.0:443").acme(
+    ///     AutoCert::builder()
+    ///         .directory_url(LETS_ENCRYPT_PRODUCTION)
+    ///         .domain("poem.rs")
+    ///         .build()
+    ///         .unwrap(),
+    /// );
+    /// ```
+    #[cfg(feature = "acme")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "acme")))]
+    #[must_use]
+    fn acme(self, auto_cert: AutoCert) -> AutoCertListener<Self>
+    where
+        Self: Sized,
+    {
+        AutoCertListener::new(self, auto_cert)
+    }
+
+    /// Wrap the listener in a `Box`.
+    fn boxed(self) -> BoxListener
+    where
+        Self: Sized + 'static,
+    {
+        Box::new(WrappedListener(self))
+    }
 }
 
+/// An owned dynamically typed Listener for use in cases where you can’t
+/// statically type your result or need to add some indirection.
+pub type BoxListener = Box<dyn Listener<Acceptor = BoxAcceptor>>;
+
 #[async_trait::async_trait]
 impl Listener for Infallible {
     type Acceptor = Infallible;
@@ -242,6 +289,20 @@ impl AsyncWrite for BoxIo {
     }
 }
 
+struct WrappedListener<T: Listener>(T);
+
+#[async_trait::async_trait]
+impl<T: Listener> Listener for WrappedListener<T>
+where
+    T::Acceptor: 'static,
+{
+    type Acceptor = BoxAcceptor;
+
+    async fn into_acceptor(self) -> IoResult<Self::Acceptor> {
+        self.0.into_acceptor().await.map(AcceptorExt::boxed)
+    }
+}
+
 struct WrappedAcceptor<T: Acceptor>(T);
 
 #[async_trait::async_trait]
